import subprocess
import os
import re
import sys
import socket
import platform
from pprint import pprint
import json
import traceback


# 适用于linux
# 简单导入 set_gpu 的一种方式:
# with open(os.environ['HOME']+'/git/mycode/tanshicheng/ad_set_gpu.py', 'r', encoding='utf-8') as r:
#     exec(r.read())
# 原文中设置显卡的地方可以避免冲突:
# if not 'CUDA_VISIBLE_DEVICES' in os.environ:
#     os.environ["CUDA_VISIBLE_DEVICES"] = "-1"

def get_public_net(analysis=True):
    """
    从不同网站获取公网ip等信息
    :param analysis: bool; 是否深入解析不同网站返回的结果, 可能网站返回形式变化导致解析错误
    :return: dict
    """
    re_ip = re.compile(r'[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}')  # 匹配ip的正则
    def have_ip_f(url):  # 传入可以 curl 的域名
        s = subprocess.getstatusoutput('curl -s --connect-timeout 3 -m 3 ' + url)[1]
        s = s if re_ip.search(s) else '{}'
        s = re.sub(r'[\r\n]+', r'\n', s).strip()
        try:
            s = json.loads(s)
        except:
            ...
        if analysis:  # 深度分析
            try:
                if url == 'myip.ipip.net':
                    s = {
                        'ip': re_ip.search(s).group(), 
                        'from': re.sub(r'[\s:：|]+', ' ', re.search('(?<=来自于).+', s).group()).strip(),
                    }
                elif url == 'ipinfo.io':
                    del s['readme']
                elif url == 'cip.cc':
                    s = {
                        'ip': re_ip.search(s).group(), 
                        'from': re.sub(r'[\s:：|]+', ' ', re.search('(?<=地址).+', s.replace('\n运营商', '')).group()).strip(),
                        'from2': re.sub(r'[\s:：|]+', ' ', re.search('(?<=数据二).+', s).group()).strip(),
                        'from3': re.sub(r'[\s:：|]+', ' ', re.search('(?<=数据三).+', s).group()).strip(),
                    }
            except:
                # traceback.print_exc()
                ...
        return s
    public_net = {  # 通过 curl 获取
        'ipip': have_ip_f('myip.ipip.net'),
        'cip': have_ip_f('cip.cc'),
        'ipinfo': have_ip_f('ipinfo.io'),
    }
    return public_net


def set_gpu(showAllGpu=False, return_more=False, public_net=False):
    """
    指定最小显存占用GPU
    :param showAllGpu: bool; 是否显示所有gpu的状态
    :param return_more: bool; 是否返回更多信息, 将以dict形式返回, 选择这个将不输出信息和设置显卡
    :param public_net: bool; 是否读取返回公网ip等信息, 需要 return_more=True
    :return:
    """
    gpu, ext_gpu_mem = -1, -1
    try:
        ext_mem = subprocess.check_output('free -m', shell=True) \
            .decode(encoding='utf8', errors='ignore') \
            .split('\n')[1].strip()
        all_mem = int(re.split(r'\s+', ext_mem)[1])
        ext_mem = int(re.split(r'\s+', ext_mem)[6])  # 不是free而是available
    except:
        ext_mem, all_mem = -1, -1
    try:
        # check_output 防止 UnicodeDecodeError
        ext_cpu = subprocess.check_output('top -bn 1 | head -n 10', shell=True) \
            .decode(encoding='utf8', errors='ignore')
        ext_cpu = re.findall(r'(?<=,)[ \d.]+?(?=id,)', ext_cpu)[0].strip()
    except:
        ext_cpu = ''
    try:
        cpu_type = subprocess.getstatusoutput('cat /proc/cpuinfo | grep name | cut -f2 -d: |uniq -c')[1].split('\n')
        cpu_type = [i.strip() for i in cpu_type]
        cpu_num = int(subprocess.getstatusoutput('cat /proc/cpuinfo |grep "physical id"|sort|uniq|wc -l')[1])
        cpu_cores = int(
            subprocess.getstatusoutput('cat /proc/cpuinfo |grep "cpu cores"|uniq')[1].split('\n')[0].split(':')[1])
        core_pro = int(subprocess.getstatusoutput('cat /proc/cpuinfo |grep "processor"|wc -l')[1])
        core_pro = int(core_pro / cpu_num / cpu_cores)
    except:
        cpu_num, cpu_cores, core_pro, cpu_type = 0, 0, 0, []
    (status, result) = subprocess.getstatusoutput('nvidia-smi')
    i_m = []
    all_gpu_mem = []
    power = 'W'
    w = []
    w_re = r'(?<=\s)[\s\d]+?W\s+?/[\s\d]+?W(?=\s)'  # 提取功率
    gpu_usage = []
    gpu_type = []
    if status == 0:
        if showAllGpu:
            print(result)
        r = re.findall(r'(?<=[|/])[\s\d]+?(?=MiB)', result)
        all_gpu_mem = [int(r[i + 1]) for i in range(0, len(r), 2)]
        r = [int(r[i + 1]) - int(r[i]) for i in range(0, len(r), 2)]
        w = [i.replace(' ', '') for i in re.findall(w_re, result)]
        gpu_usage = [int(re.search(r'(?<= )[0-9.]+?(?=%)', i.split('MiB', 1)[1]).group())
                     for i in result.split('\n') if re.search(w_re, i)]
        i_m = [(i, j) for i, j in enumerate(r)]
        if i_m:
            i_m_1 = sorted(i_m, key=lambda t: t[1])[-1]
            gpu = i_m_1[0]
            ext_gpu_mem = i_m_1[1]
        if w:
            power = w[gpu]
        # 每个gpu型号获取
        # result_L = result.split('\n')  # 这种方式获取不全
        # for i in range(1, len(result_L)):
        #     if re.search(r'\|\s*?\d+?MiB / \d+?MiB \|', result_L[i]):
        #         gpu_type.append(re.split(r' [ ]+', result_L[i - 1])[2])
        gpu_type = subprocess.getstatusoutput('nvidia-smi -L')[1].split('\n')
        gpu_type = [re.search(r'(?<=: )[^(\r\n]+', i).group().strip() for i in gpu_type]
    hostname = subprocess.getstatusoutput('hostname')[1]
    ip = socket.gethostbyname(hostname)
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    except:
        ...
    if return_more:
        n = int(cpu_num * cpu_cores * core_pro)
        ret = {
            'hostname': hostname,  # str, 主机名
            'ip': ip,  # str; 主机局域网ip
            'gpu_power': w,  # list; 每张gpu的使用功率和上限功率
            'gpu_usage': gpu_usage,  # list; 每张gpu的使用率
            'ext_gpu_mem': [i[1] for i in i_m],  # list; 每张gpu的剩余显存数量, 单位MB
            'all_gpu_mem': all_gpu_mem,  # list; 每张gpu的总显存数量, 单位MB
            'ext_cpu': ext_cpu,  # str; 剩余的cpu使用率, 单位百分比
            'ext_mem': ext_mem,  # int; 剩余内存数量, 单位MB
            'all_mem': all_mem,  # int; 总计内存数量, 单位MB
            'cpu_info': f"{n}:{cpu_num}-{cpu_cores}-{core_pro}",  # str; 总线程数:cpu数-每个cpu的核心数-每个核心的超线程数
            'platform': platform.platform(),  # str; 平台架构, 比如 Darwin-20.4.0-arm64-arm-64bit
            'gpu_type': gpu_type,  # list; 每个gpu的型号, 比如 NVIDIA GeForce RTX 2080 Ti
            'cpu_type': cpu_type,  # list; 每个cpu的型号, 比如 16  Intel(R) Core(TM) i9-9900K CPU @ 3.60GHz
        }
        if public_net:
            ret['public_net'] = get_public_net()
        return ret
    # 例如: tsc@192.168.6.6, 设置显卡:0(28W/260W)-0, 剩余CPU:%(1-8-2), 剩余内存:57.7GB, 剩余显存:10.8GB
    print('%s@%s, 设置显卡:%d(%s)-%d, 剩余CPU:%s%%(%d-%d-%d), 剩余内存:%.1fGB, 剩余显存:%.1fGB' %
          (hostname, ip, gpu, power, len(i_m) - 1, ext_cpu, cpu_num, cpu_cores, core_pro, ext_mem / 1024,
           ext_gpu_mem / 1024))
    os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
    os.environ["CUDA_VISIBLE_DEVICES"] = str(gpu)
    return gpu  # 选择的显存最大的gpu编号


if __name__ == '__main__':
    gpu = set_gpu()
    if len(sys.argv) > 1:
        sys.exit(gpu + 1)
    else:
        ret = set_gpu(return_more=True, public_net=True)
        # pprint(ret)
        print(json.dumps(ret, ensure_ascii=False, indent=2))



import os
import sys
import subprocess
import re
import time
from datetime import datetime
from pprint import pprint
try:
    from set_gpu import set_gpu
    from kill import get_user_processes
    from nvidia_htop import nvidia_htop
except:
    from .set_gpu import set_gpu
    from .kill import get_user_processes
    from .nvidia_htop import nvidia_htop


def get_current_user_cmd():
    username = subprocess.getstatusoutput('whoami')[1]
    user_processes = get_user_processes()
    if username not in user_processes:
        print('没有相关用户进程', username)
        return
    processes = user_processes[username]['pro']
    processes = sorted(processes, key=lambda t: t['command'])
    all_cmd = set()
    print('PID\tCommand')
    for i, p in enumerate(processes):
        if p['command'] in all_cmd:
            continue
        print(p['pid'], p['command'], sep='\t')
        all_cmd.add(p['command'])
    print('总共 {} 个程序, {} 种程序'.format(len(processes), i))


def benchmark():
    print('显卡(cuda:0)性能测试: cuda 11.7 下 3090Ti=85(理论160), V100 16GB=86(理论125), A100 80GB=231(理论312); cuda 11.4 下 2080Ti=58')
    import torch
    from torch.utils import benchmark
    typ = torch.float16
    n = 1024 * 16
    a = torch.randn(n, n).type(typ).cuda()
    b = torch.randn(n, n).type(typ).cuda()
    t = benchmark.Timer(stmt='a @ b', globals={'a': a, 'b': b})
    x = t.timeit(50)
    print(2*n**3 / x.median / 1e12)


def wait_gpus(para_L):
    args = {
        '-c': 0.1,  # 至少剩余多少gpu使用率, 百分比
        '-m': 0.8,  # 至少剩余多少显存, 小于1是百分比, 大于1是MB
        '-t': 10,  # 这些显卡至少连续几秒钟满足这些条件
        '-g': '',  # 等待哪些显卡,例如: 0,2
    }
    i = 0
    for p in para_L:
        if '=' in p and p.split('=')[0] in args:
            if p.split('=')[0] in {'-g'}:
                args[p.split('=')[0]] = p.split('=')[1]
            else:
                args[p.split('=')[0]] = float(p.split('=')[1])
        else:
            break
        i += 1
    para_L = para_L[i:]
    cmd = ' ' + ' '.join(para_L) + ' '
    # 获取gpus
    gpus = [int(i) for i in args['-g'].split(',') if i]
    # gpus = []
    # gpus_re = [
    #     r'(?<= --include=localhost:)\d[\d,]*(?= )',  # deepspeed
    #     r'(?<= CUDA_VISIBLE_DEVICES=)\d[\d,]*(?= )',
    # ]
    # for r in gpus_re:
    #     ret = re.search(r, cmd)
    #     if ret:
    #         gpus = [int(i) for i in ret.group().strip(',').split(',')]
    #         break
    # 等待命令
    print(cmd.strip())
    print('等待GPU {} 满足条件:'.format(gpus), args, datetime.now())
    gpu_info = {i: None for i in gpus}  # {序号:满足要求的时间,..}
    start = time.time()
    while True:
        ret = set_gpu(return_more=True)
        for g in gpu_info.keys():
            gpu_usage = ret['gpu_usage'][g]
            ext_gpu_mem = ret['ext_gpu_mem'][g]
            all_gpu_mem = ret['all_gpu_mem'][g]
            mem_rate = args['-m'] / all_gpu_mem if args['-m'] > 1 else args['-m']
            if (100 - gpu_usage) / 100 >= args['-c'] and ext_gpu_mem / all_gpu_mem >= mem_rate:
                gpu_info[g] = time.time() if gpu_info[g] is None else gpu_info[g]
            else:
                gpu_info[g] = None
        bypass = False if gpu_info else True
        for t in gpu_info.values():
            if t is not None and time.time() - t > args['-t']:
                bypass = True
            else:
                bypass = False
                break
        if bypass:
            print('等待 {} 秒GPU满足条件开始运行'.format(round(time.time()-start, 2)), datetime.now())
            os.system(cmd.strip())
            break
        time.sleep(1)
        
        
def show_stat(width=100):
    ret = set_gpu(return_more=True, public_net=False)
    # 整合gpu信息
    gpu_info = ['gpu_usage', 'ext_gpu_mem', 'all_gpu_mem', 'gpu_power', 'gpu_type']
    if sum(g in ret for g in gpu_info) == len(gpu_info):
        ret['gpu_usage_mem_power_type'] = []
        for i, (u, m, am, p, t) in enumerate(zip(*[ret[g] for g in gpu_info])):
            ret['gpu_usage_mem_power_type'].append((i, u, '{}/{}M'.format(am-m, am), p, t))
        for g in gpu_info:
            del ret[g]
    # 整合cpu信息
    cpu_info = ['ext_cpu', 'ext_mem', 'all_mem', 'cpu_info']
    if sum(g in ret for g in cpu_info) == len(cpu_info):
        u, m, am, t = [ret[g] for g in cpu_info]
        ret['cpu_usage_mem_num'] = [round(100 - float(u), 1), '{}/{}M'.format(am-m, am), t]
        for g in cpu_info:
            del ret[g]
    # 整合网络信息
    net_info = ['hostname', 'ip']
    if sum(g in ret for g in net_info) == len(net_info):
        ret['hostname_ip'] = [ret[g] for g in net_info]
        for g in net_info:
            del ret[g]
    ret = sorted(ret.items(), key=lambda t: t[0])
    pprint(ret, width=width)
    nvidia_htop(l=width-62, c=False, p=None, show_gpu=False)


def main():
    para = sys.argv[1:]
    if para:
        if para[0] == '--wait':  # 一个等待gpu的功能
            wait_gpus(para[1:])
            return
        elif para[0] == '--benchmark':  # 测试显卡性能
            benchmark()
            return
        elif para[0] == '--showp':  # 显示当前用户正在运行的命令(去重复)
            get_current_user_cmd()
            return
        elif re.search(r'^--show($|[= ]+[0-9]+$)', para[0]):  # 显示当前系统资源信息
            width = re.split(r'[= ]+', para[0])
            if len(width) == 2:
                show_stat(int(width[1]))
            else:
                show_stat()
            return
    # 一些转义符号复原
    for i, p in enumerate(para):
        if '\\' in p:
            para[i] = para[i].replace('\\', '\\\\')
        if '"' in p:
            para[i] = '"' + para[i].replace('"', '\\"') + '"'
        if '$' in p:
            para[i] = para[i].replace('$', '\\\\\\$')  # 因为 shell eval 所以多加入转义
    # 自动寻找 auto.sh 的位置
    auto = subprocess.getstatusoutput('py=$(which python) && echo ${py%bin*}lib/python*/site-packages/tsc_auto/auto.sh')
    # 没有报错
    if auto[0] == 0:
        auto = auto[1]
    else:
        raise NameError(str(auto) + ' 寻找auto.sh错误!')
    # 运行
    cmd = f'chmod 777 {auto} ; {auto} ' + ' '.join(para)
    os.system(cmd)


if __name__ == '__main__':
    main()



'''
### 暂时无用的用于超算的文件
- info.py
- query.py
- srun.sh
'''
from .set_gpu import set_gpu, get_public_net
from .kill import get_nvidia_processes, get_ps_processes, get_user_processes, kill_processes
from .notice import send_wechat, get_time_diff, explain_exit_code
from .nvidia_htop import nvidia_htop


import argparse
import os
import time


def main():
    parser = argparse.ArgumentParser(description='断线自动重连ssh, 格式: ressh [自带参数] ssh参数',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('--tmux', action='store_true', help='是否自动进入tmux')
    parser.add_argument('--password', type=str, default=None, help='ssh连接的密码(先正常登录一次)')
    parser.add_argument('--try_interval', type=float, default=3, help='断线重连接的间隔时间,单位秒')
    parser.add_argument('--tp', action='store_true', help='用于协助默认端口参数下的tp代理命令')
    parser.add_argument('--tp_r', type=str, default='24943:127.0.0.1:7890', help='tp远程端口转发信息')
    args, cmd_ = parser.parse_known_args()
    # 构建命令
    cmd_ = ['-o ServerAliveInterval=5 -o ServerAliveCountMax=3'] + cmd_
    if args.tp and args.tp_r:
        cmd_ = ['-R {}'.format(args.tp_r)] + cmd_
    if args.tmux:
        if args.password:
            cmd_ = ['-t -o RemoteCommand=tmux\\ a||~/tmux\\ a||tmux||~/tmux'] + cmd_  # 或者加双引号放在最后并去除 -o RemoteCommand
        else:
            cmd_ = ['-t -o RemoteCommand="tmux a||~/tmux a||tmux||~/tmux"'] + cmd_
    cmd = 'ssh ' + ' '.join(cmd_)
    if args.password:
        cmd = """expect -c 'spawn {}; expect "*password:"; send "{}\\r"; interact'""".format(cmd, args.password)
    print(cmd)
    # 循环运行
    i_try = 0
    while True:
        i_try += 1
        if os.system(cmd):
            print('第 {} 次中断, {} 秒后重新连接...'.format(i_try, args.try_interval))
            time.sleep(args.try_interval)
        else:
            break


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

#######
# from https://github.com/peci1/nvidia-htop/blob/master/nvidia-htop.py
# USAGE
#
# [nvidia-smi | ] nvidia-htop.py [-l [length]]
#   print GPU utilization with usernames and CPU stats for each GPU-utilizing process
#
#   -l|--command-length [length]     Print longer part of the commandline. If `length'
#                                    is provided, use it as the commandline length,
#                                    otherwise print first 100 characters.
#   -c|--color                       Colorize the output (green - free GPU, yellow -
#                                    moderately used GPU, red - fully used GPU)
######

import sys
import os
import re
import subprocess
import select
import argparse
from pip import main

MEMORY_FREE_RATIO = 0.05
MEMORY_MODERATE_RATIO = 0.9
GPU_FREE_RATIO = 0.05
GPU_MODERATE_RATIO = 0.75


def nvidia_htop(l=18, c=False, p=None, show_gpu=True):
    # parser = argparse.ArgumentParser()
    # parser.add_argument('-l', '--command-length', default=20, const=100, type=int, nargs='?')
    # parser.add_argument('-c', '--color', action='store_true')
    # # only for testing
    # parser.add_argument('-p', '--fake-ps', help="The list of processes to use instead of real output of `ps`")

    # args = parser.parse_args()

    # # parse the command length argument
    # command_length = args.command_length
    # color = args.color
    # fake_ps = args.fake_ps
    
    command_length = l
    color = c
    fake_ps = p

    # for testing, the stdin can be provided in a file
    fake_stdin_path = os.getenv("FAKE_STDIN_PATH", None)
    stdin_lines = []
    if sys.stdin in select.select([sys.stdin], [], [], 0)[0]:
        stdin_lines = sys.stdin.readlines()

    if fake_stdin_path is not None:
        with open(fake_stdin_path, 'rt') as f:
            lines = f.readlines()
    elif stdin_lines:
        lines = stdin_lines
    else:
        ps_call = subprocess.run('nvidia-smi', stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if ps_call.returncode != 0:
            print('nvidia-smi exited with error code {}:'.format(ps_call.returncode))
            print(ps_call.stdout.decode() + ps_call.stderr.decode())
            sys.exit()
        lines_proc = ps_call.stdout.decode().split("\n")
        lines = [line + '\n' for line in lines_proc[:-1]]
        lines += lines_proc[-1]

    def colorize(_lines):
        from termcolor import colored  # 有依赖
        for j in range(len(_lines)):
            line = _lines[j]
            m = re.match(r"\| (?:N/A|..%)\s+[0-9]{2,3}C.*\s([0-9]+)MiB\s+/\s+([0-9]+)MiB.*\s([0-9]+)%", line)
            if m is not None:
                used_mem = int(m.group(1))
                total_mem = int(m.group(2))
                gpu_util = int(m.group(3)) / 100.0
                mem_util = used_mem / float(total_mem)

                is_moderate = False
                is_high = gpu_util >= GPU_MODERATE_RATIO or mem_util >= MEMORY_MODERATE_RATIO
                if not is_high:
                    is_moderate = gpu_util >= GPU_FREE_RATIO or mem_util >= MEMORY_FREE_RATIO

                c = 'red' if is_high else ('yellow' if is_moderate else 'green')
                _lines[j] = colored(_lines[j], c)
                _lines[j-1] = colored(_lines[j-1], c)

        return _lines

    lines_to_print = []
    is_new_format = False
    # Copy the utilization upper part verbatim
    for i in range(len(lines)):
        if not lines[i].startswith("| Processes:"):
            lines_to_print.append(lines[i].rstrip())
        else:
            while not lines[i].startswith("|===="):
                m = re.search(r'GPU\s*GI\s*CI', lines[i])
                if m is not None:
                    is_new_format = True
                i += 1
            i += 1
            break

    if color:
        lines_to_print = colorize(lines_to_print)

    # we print all but the last line which is the +---+ separator
    if show_gpu:
        for line in lines_to_print[:-1]:
            print(line)

    no_running_process = "No running processes found"
    if no_running_process in lines[i] or lines[i].startswith("+--"):
        print(lines[-1].strip())
        print("| " + no_running_process + " " * (73 - len(no_running_process)) + "   |")
        # Issue #9, running inside docker and seeing no processes
        if lines[i].startswith("+--"):
            print("| If you're running in a container, you'll only see processes running inside. |")
        print(lines[-1])
        sys.exit()

    # Parse the PIDs from the lower part
    gpu_num = []
    pid = []
    gpu_mem = []
    user = []
    cpu = []
    mem = []
    time = []
    command = []

    gpu_num_idx = 1
    pid_idx = 2 if not is_new_format else 4
    gpu_mem_idx = -3

    while not lines[i].startswith("+--"):
        if "Not Supported" in lines[i]:
            i += 1
            continue
        line = lines[i]
        line = re.split(r'\s+', line)
        gpu_num.append(line[gpu_num_idx])
        pid.append(line[pid_idx])
        gpu_mem.append(line[gpu_mem_idx])
        user.append("")
        cpu.append("")
        mem.append("")
        time.append("")
        command.append("")
        i += 1

    if fake_ps is None:
        # Query the PIDs using ps
        ps_format = "pid,user,%cpu,%mem,etime,command"
        ps_call = subprocess.run(["ps", "-o", ps_format, "-p", ",".join(pid)], stdout=subprocess.PIPE)
        processes = ps_call.stdout.decode().split("\n")
    else:
        with open(fake_ps, 'r') as f:
            processes = f.readlines()

    # Parse ps output
    for line in processes:
        if line.strip().startswith("PID") or len(line) == 0:
            continue
        parts = re.split(r'\s+', line.strip(), 5)
        # idx = pid.index(parts[0])
        for idx in filter(lambda p: pid[p] == parts[0], range(len(pid))):
            user[idx] = parts[1]
            cpu[idx] = parts[2]
            mem[idx] = parts[3]
            time[idx] = parts[4] if "-" not in parts[4] else parts[4].split("-")[0] + " days"
            command[idx] = parts[5]

    max_pid_length = max(5, max([len(x) for x in pid]))
    format = ("|  %3s %" + str(max_pid_length) + "s %8s   %8s %5s %5s %9s  %-" +
              str(command_length) + "." + str(command_length) + "s  |")

    line = format % (
        "GPU", "PID", "USER", "GPU MEM", "%CPU", "%MEM", "TIME", "COMMAND"
    )

    print("+" + ("-" * (len(line) - 2)) + "+")

    print(line)

    for i in range(len(pid)):
        print(format % (
            gpu_num[i],
            pid[i],
            user[i],
            gpu_mem[i],
            cpu[i],
            mem[i],
            time[i],
            command[i]
        ))

    print("+" + ("-" * (len(line) - 2)) + "+")


if __name__ == '__main__':
    nvidia_htop()



import argparse
import os
import subprocess


def main():
    parser = argparse.ArgumentParser(description='代理工具',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('--tp__port', type=int, default=None, help='访问服务器的转发代理端口,默认24943')
    parser.add_argument('--tp__ip', type=str, default=None, help='访问服务器的转发代理ip,默认127.0.0.1')
    parser.add_argument('--tp__socks5', action='store_true', help='是否使用socks5协议,默认http')
    args, cmd_ = parser.parse_known_args()
    # cmd_ 空格处理
    for i, v in enumerate(cmd_):
        if ' ' in v and '"' not in v:
            cmd_[i] = '"{}"'.format(v)
    # 自动寻找 proxychains4 的位置
    pc4 = subprocess.getstatusoutput(
        'py=$(which python) && echo ${py%bin*}lib/python*/site-packages/tsc_auto/proxychains4')
    # 没有报错
    if pc4[0] == 0:
        pc4 = pc4[1]
        pc4_conf = os.path.join(os.path.dirname(pc4), 'proxychains.config')
        pc4_so = os.path.join(os.path.dirname(pc4), 'libproxychains4.so')
    else:
        raise NameError(str(pc4) + ' 寻找proxychains4错误!')
    # 运行
    cmd = 'chmod 777 {} {} ; {} -f {} '.format(pc4, pc4_so, pc4, pc4_conf) + ' '.join(cmd_)
    if args.tp__port and args.tp__port != 24943 or args.tp__ip and args.tp__ip != '127.0.0.1' or args.tp__socks5:
        cmd_add = 'cp {} ~/.proxychains.config && '.format(pc4_conf)
        if args.tp__port:
            cmd_add += 'sed -i "s/24943/{}/g" ~/.proxychains.config && '.format(args.tp__port)
        if args.tp__ip:
            cmd_add += 'sed -i "s/127.0.0.1/{}/g" ~/.proxychains.config && '.format(args.tp__ip)
        if args.tp__socks5:
            cmd_add += 'sed -i "s/http/socks5/g" ~/.proxychains.config && '
        cmd = cmd_add + cmd.replace(pc4_conf, '~/.proxychains.config')
        cmd += '; rm ~/.proxychains.config'
    os.system(cmd)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os

if os.path.exists('readme.md'):
    long_description = open('readme.md', 'r', encoding='utf8').read()
else:
    long_description = '教程: https://github.com/aitsc/tsc-auto'

setup(
    name='tsc-auto',
    version='0.46',
    description="Auto selector for GPU and CUDA, support the detection of tensorflow or torch",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='tanshicheng',
    license='GPLv3',
    url='https://github.com/aitsc/tsc-auto',
    keywords='tools',
    packages=find_packages(),
    include_package_data=True,  # 包含 .sh 文件
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Software Development :: Libraries',
    ],
    entry_points={  # 打包到bin
        'console_scripts': [
            'ta=tsc_auto.auto:main',  # 包不能有-符号
            'tkill=tsc_auto.kill:main',
            'ressh=tsc_auto.ressh:main',
            'tp=tsc_auto.proxy:main',
        ],
    },
    python_requires='>=3.6',
)
